/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.common.ui.javafx.control.treetableview;

import javafx.scene.control.TextField;
import javafx.scene.control.TreeTableRow;
import javafx.scene.control.cell.TextFieldTreeTableCell;
import javafx.scene.input.KeyCode;
import javafx.util.StringConverter;
import javafx.util.converter.DefaultStringConverter;

/**
 * A DynamicTextFieldTreeTableCell that can dynamically decide per element,
 * whether it should be editable.
 * 
 * Also the editCancel behavior can be configured for cases in which the cell
 * only loses the selection.
 * 
 * @author bayha
 */
public class DynamicTextFieldTreeTableCell<T> extends TextFieldTreeTableCell<T, String> {

	/** Flag to indicate any occurred Escape key press events during editing. */
	private boolean escapePressed = false;

	/** The currently edited Element. */
	private T currentlyEditedElement;

	/** The current entered text in the TextField. */
	private String currentInput = "";

	/**  */
	DynamicTreeTableUIProviderBase<T> uiProvider;

	/** The column index, this cell is embedded in. */
	int columnIndex;

	/**
	 * Constructor.
	 */
	public DynamicTextFieldTreeTableCell(DynamicTreeTableUIProviderBase<T> uiProvider,
			int columnIndex) {
		this(new DefaultStringConverter());

		this.uiProvider = uiProvider;
		this.columnIndex = columnIndex;
	}

	/**
	 * Constructor.
	 */
	public DynamicTextFieldTreeTableCell(StringConverter<String> converter) {
		super(converter);

		this.setOnKeyPressed(event -> {
			if(event.getCode().equals(KeyCode.ESCAPE)) {
				if(currentlyEditedElement != null) {
					escapePressed = true;
				}
			}
		});
	}

	/**
	 * May be overwritten to determine for this cell, whether it is editable for a
	 * given element.
	 * 
	 * @param element
	 *            The element for which to decide, whether it shall be editable.
	 * @return true, if the given element is supposed to be editable. Otherwise false.
	 */
	protected boolean canEdit(T element) {
		return true;
	}

	/**
	 * May be overwritten, if the current input value is supposed to be kept/set
	 * after leaving the cell, e.g. by selecting a different cell or view.
	 * 
	 * Note, that with this, only pressing Escape will cancel an editing operation
	 * of a user.
	 * 
	 * @param element
	 *            The element to set the current text input as new value.
	 * @param currentValue
	 *            The current value from the editing process.
	 */
	protected void setValueOnLeavingCell(T element, String currentValue) {
		// Empty
	}

	/** {@inheritDoc} */
	@Override
	public void startEdit() {
		TreeTableRow<T> row = getTreeTableRow();
		T currentEntry = row.getItem();

		if(canEdit(currentEntry)) {
			super.startEdit();

			// Remember currently edited element.
			currentlyEditedElement = currentEntry;

			String initialEditorContent =
					uiProvider.getInitialEditorContent(currentlyEditedElement, columnIndex);

			// Reset currentInput buffer for the new element.
			currentInput = initialEditorContent;

			// We keep the current input in the buffer variable currentInput to be able to
			// use it during unintended cancel options. See cancelEdit().
			TextField textField = (TextField)getGraphic();

			if(textField != null) {
				textField.setOnKeyReleased(event -> {
					currentInput = textField.getText();
				});

				textField.setText(initialEditorContent);
			}
		} else {
			// Reset buffer
			currentlyEditedElement = null;
			currentInput = null;
		}
	}

	/** {@inheritDoc} */
	@Override
	public void cancelEdit() {
		super.cancelEdit();

		if(escapePressed) {
			escapePressed = false;
			currentlyEditedElement = null;
		} else {
			// Reset the new value to this cell.
			setText(currentInput);

			// Set the current TextField value if intended.
			if(currentlyEditedElement != null) {
				String initialEditorContent =
						uiProvider.getInitialEditorContent(currentlyEditedElement, columnIndex);
				if(!currentInput.equals(initialEditorContent)) {
					setValueOnLeavingCell(currentlyEditedElement, currentInput);
				}
				currentlyEditedElement = null;
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void commitEdit(String newValue) {
		// Reset the currentlyEditedElement since the editing is over.
		currentlyEditedElement = null;

		super.commitEdit(newValue);
	}
}
